@@ -50,7 +50,7 @@ class PAOFLOW:
 
 
 
-  def __init__ ( self, workpath='./', outputdir='output', inputfile=None, savedir=None, model=None, npool=1, smearing='gauss', acbn0=False, verbose=False, restart=False ):
+  def __init__ ( self, workpath='./', outputdir='output', inputfile=None, savedir=None, model=None, npool=1, smearing='gauss', acbn0=False, verbose=False, restart=False):
     '''
     Initialize the PAOFLOW class, either with a save directory with required QE output or with an xml inputfile
     Arguments:
@@ -144,7 +144,7 @@ class PAOFLOW:
       # Format string and print
       lms = spaces-lmn
       dt = time() - self.reset_time
-      print('%s in: %s %8.3f sec'%(mname,lms*' ',dt))
+      print('%s in: %s %8.3f sec'%(mname,lms*' ',dt), flush=True)
       self.reset_time = time()
 
 
@@ -259,6 +259,79 @@ class PAOFLOW:
 
 
 
+  def projections ( self, internal=False ):
+    '''
+    Calculate the projections on the atomic basis provided by the pseudopotential or 
+    on the all-electron internal basis sets.
+    Replaces projwfc.
+    TODO  * add spin-orbit and non-collinear cases
+    '''
+
+    from .defs.do_atwfc_proj import build_pswfc_basis_all
+    from .defs.do_atwfc_proj import build_aewfc_basis
+    from .defs.do_atwfc_proj import calc_proj_k
+    from .defs.communication import load_balancing, gather_array
+    
+    arry,attr = self.data_controller.data_dicts()
+    
+    if internal:
+      basis,arry['shells'] = build_aewfc_basis(self.data_controller)
+    else:
+      basis,arry['shells'] = build_pswfc_basis_all(self.data_controller)
+    nkpnts = len(arry['kpnts'])
+    nbnds = attr['nbnds']
+    nspin = attr['nspin']
+    natwfc = len(basis)
+    attr['nawf'] = natwfc
+    
+    ini_ik,end_ik = load_balancing(self.size, self.rank,nkpnts)
+    Unewaux = np.zeros((end_ik-ini_ik,nbnds,natwfc,nspin), dtype=complex)
+    for ispin in range(nspin):
+      for ik in range(ini_ik,end_ik):
+        Unewaux[ik-ini_ik,:,:,ispin] = calc_proj_k(self.data_controller, basis, ik, ispin)
+    
+    Unew = np.zeros((nkpnts,nbnds,natwfc,nspin), dtype=complex) if self.rank == 0 else None
+    gather_array(Unew,Unewaux)
+    if self.rank == 0: Unew = np.moveaxis(Unew,0,2)
+    Unew = self.comm.bcast(Unew, root=0)
+    
+    arry['U'] = Unew
+    arry['basis'] = basis
+    
+    self.report_module_time('Projections')
+
+
+
+  def read_atomic_proj_QE ( self ):
+    '''
+      Read the wavefunctions and overlaps from atomic-proj.xml, written by Quantum Espresso
+      in the .save directory specified in PAOFLOW's constructos. They are saved to the 
+      DataController's arrays dictionary with keys 'U' and 'Sks', respectively.
+    '''
+    from .defs.read_upf import UPF
+    from os.path import exists,join
+
+    arry,attr = self.data_controller.data_dicts()
+    fpath = attr['fpath']
+    if exists(join(fpath,'atomic_proj.xml')):
+      from .defs.read_QE_xml import parse_qe_atomic_proj
+      parse_qe_atomic_proj(self.data_controller, join(fpath,'atomic_proj.xml'))
+    else:
+      raise Exception('atomic_proj.xml was not found.\n')
+
+    arry['jchia'] = {}
+    arry['shells'] = {}
+    for at,pseudo in arry['species']:
+      fname = join(attr['fpath'], pseudo)
+      if exists(fname):
+        upf = UPF(fname)
+        arry['shells'][at] = upf.shells
+        arry['jchia'][at] = upf.jchia
+      else:
+        raise Exception('Pseudopotential not found: %s'%fname)
+
+
+
   def projectability ( self, pthr=0.95, shift='auto' ):
     '''
     Calculate the Projectability Matrix to determine how many states need to be shifted
@@ -279,19 +352,19 @@ class PAOFLOW:
 
     try:
       do_projectability(self.data_controller)
-    except:
+    except Exception as e:
       self.report_exception('projectability')
       if attr['abort_on_exception']:
-        self.comm.Abort()
+        raise e
 
     self.report_module_time('Projectability')
+    
 
 
-
-  def pao_hamiltonian ( self, shift_type=1, write_binary=False, expand_wedge=True, symmetrize=False, thresh=1.e-6, max_iter=16 ):
+  def pao_hamiltonian ( self, shift_type=1, insulator=False, write_binary=False, expand_wedge=True, symmetrize=False, thresh=1.e-6, max_iter=16 ):
     '''
     Construct the Tight Binding Hamiltonian
-    Yields 'HRs', 'Hks' and 'kq_wght'
+    Populates DataController with 'HRs', 'Hks' and 'kq_wght'
 
     Arguments:
         shift_type (int): Shift type [ 0-(PRB 2016), 1-(PRB 2013), 2-No Shift ] 
@@ -302,10 +375,12 @@ class PAOFLOW:
     '''
     from .defs.get_K_grid_fft import get_K_grid_fft
     from .defs.do_build_pao_hamiltonian import do_build_pao_hamiltonian,do_Hks_to_HRs
+    from .defs.do_Efermi import E_Fermi
 
     # Data Attributes and Arrays
     arrays,attr = self.data_controller.data_dicts()
 
+    if insulator: attr['insulator'] = True
     if 'shift_type' not in attr: attr['shift_type'] = shift_type
     if 'write_binary' not in attr: attr['write_binary'] = write_binary
 
@@ -320,12 +395,10 @@ class PAOFLOW:
 
     try:
       do_build_pao_hamiltonian(self.data_controller)
-    except SystemExit as se:
-      quit()
-    except:
+    except Exception as e:
       self.report_exception('pao_hamiltonian')
       if attr['abort_on_exception']:
-        self.comm.Abort()
+        raise e
     self.report_module_time('Building Hks')
 
     # Done with U and Sks
@@ -338,10 +411,10 @@ class PAOFLOW:
       self.data_controller.broadcast_single_array('HRs')
 
       get_K_grid_fft(self.data_controller)
-    except:
+    except Exception as e:
       self.report_exception('pao_hamiltonian')
       if attr['abort_on_exception']:
-        self.comm.Abort()
+        raise e
     self.report_module_time('k -> R')
 
 
@@ -374,10 +447,10 @@ class PAOFLOW:
         add_ext_field(self.data_controller)
         if self.rank == 0 and attr['verbose']:
           print('External Fields Added')
-    except:
+    except Exception as e:
       self.report_exception('add_external_fields')
       if attr['abort_on_exception']:
-        self.comm.Abort()
+        raise e
 
     self.comm.Barrier()
     
@@ -395,10 +468,10 @@ class PAOFLOW:
     '''
     try:
       self.data_controller.write_z2pack(fname)
-    except:
+    except Exception as e:
       self.report_exception('z2_pack')
       if self.data_controller.data_attributes['abort_on_exception']:
-        self.comm.Abort()
+        raise e
 
 
   def bands ( self, ibrav=None, band_path=None, high_sym_points=None, spin_orbit=False, fname='bands', nk=500 ):
@@ -448,10 +521,10 @@ class PAOFLOW:
       E_kp = gather_full(arrays['E_k'], attr['npool'])
       self.data_controller.write_bands(fname, E_kp)
       E_kp = None
-    except:
+    except Exception as e:
       self.report_exception('bands')
       if attr['abort_on_exception']:
-        self.comm.Abort()
+        raise e
 
     self.report_module_time('Bands')
 
@@ -511,10 +584,10 @@ mo    '''
 
     try:
       wave_function_site_projection(self.data_controller)
-    except:
+    except Exception as e:
       self.report_exception('wave_function_projection')
       if self.data_controller.data_attributes['abort_on_exception']:
-        self.comm.Abort()
+        raise e
 
     self.report_module_time('wave_function_projection')
 
@@ -539,10 +612,10 @@ mo    '''
     
     try:
       doubling_HRs(self.data_controller)
-    except:
+    except Exception as e:
       self.report_exception('doubling_Hamiltonian')
       if attr['abort_on_exception']:
-        self.comm.Abort()
+        raise e
 
     self.report_module_time('doubling_Hamiltonian')
   
@@ -575,10 +648,10 @@ mo    '''
 
       _,_,attr['nk1'],attr['nk2'],attr['nk3'],_ = arry['HRs'].shape
       attr['nkpnts'] = attr['nk1']*attr['nk2']*attr['nk3']
-    except:
+    except Exception as e:
       self.report_exception('cutting_Hamiltonian')
       if attr['abort_on_exception']:
-        self.comm.Abort()
+        raise e
 
 
 
@@ -604,8 +677,17 @@ mo    '''
 
     if ('sh_l' not in arrays and 'sh_j' not in arrays) and not adhoc_SO:
       if sh_l is None and sh_j is None:
-        from .defs.read_sh_nl import read_sh_nl
-        arrays['sh_l'],arrays['sh_j'] = read_sh_nl(self.data_controller)
+        sh = arrays['shells']
+        shells,jchia = [],[]
+        for i,a in enumerate(arrays['atoms']):
+          ash = []
+          for v in sh[a]:
+            ash += [v, v] if v==0 else [v]
+          shells += ash[::2]
+          for l in ash[::2]:
+            jchia += [.5, .5] if l==0 else [l-.5, l+.5]
+        arrays['sh_j'] = np.array(jchia)[::2]
+        arrays['sh_l'] = np.array(shells)
       else:
         arrays['sh_l'],arrays['sh_j'] = sh_l,sh_j
     try:
@@ -631,11 +713,10 @@ mo    '''
           Sj[spol,:,:] = clebsch_gordan(nawf, arrays['sh_l'], arrays['sh_j'], spol)
 
       arrays['Sj'] = Sj
-    except:
+    except Exception as e:
       self.report_exception('spin_operator')
       if attr['abort_on_exception']:
-        self.comm.Abort()
-
+        raise e
 
 
 
@@ -666,9 +747,9 @@ mo    '''
     if 'spin_Hall' not in attr: attr['spin_Hall'] = spin_Hall
     if 'do_spin_orbit' not in attr: attr['do_spin_orbit'] = spin_orbit
 
-    if 'spol' not in attr: attr['spol'] = spol
-    if 'ipol' not in attr: attr['ipol'] = ipol
-    if 'jpol' not in attr: attr['jpol'] = jpol
+    attr['spol'] = spol
+    attr['ipol'] = ipol
+    attr['jpol'] = jpol
 
     if attr['spol'] is None or attr['ipol'] is None or attr['jpol'] is None:
       if self.rank == 0:
@@ -680,10 +761,10 @@ mo    '''
 
     try:
       do_topology(self.data_controller)
-    except:
+    except Exception as e:
       self.report_exception('topology')
       if attr['abort_on_exception']:
-        self.comm.Abort()
+        raise e
 
     self.report_module_time('Band Topology')
 
@@ -694,10 +775,10 @@ mo    '''
 
 
 
-  def interpolated_hamiltonian ( self, nfft1=0, nfft2=0, nfft3=0 ):
+  def interpolated_hamiltonian ( self, nfft1=0, nfft2=0, nfft3=0, reshift_Ef=False ):
     '''
     Calculate the interpolated Hamiltonian with the method of zero padding
-    Yields 'Hksp'
+    Populates DataController with 'Hksp'
 
     Arguments:
         nfft1 (int): Desired size of the interpolated Hamiltonian's first dimension
@@ -710,6 +791,7 @@ mo    '''
     from .defs.get_K_grid_fft import get_K_grid_fft
     from .defs.do_double_grid import do_double_grid
     from .defs.communication import gather_scatter
+    from .defs.do_Efermi import E_Fermi
 
     arrays,attr = self.data_controller.data_dicts()
 
@@ -742,8 +824,16 @@ mo    '''
       snawf,_,_,_,nspin = arrays['Hksp'].shape
       arrays['Hksp'] = np.reshape(arrays['Hksp'], (snawf,attr['nkpnts'],nspin))
       arrays['Hksp'] = gather_scatter(arrays['Hksp'], 1, attr['npool'])
+
       snktot = arrays['Hksp'].shape[1]
-      arrays['Hksp'] = np.reshape(np.moveaxis(arrays['Hksp'],0,1), (snktot,nawf,nawf,nspin))
+      if reshift_Ef:
+        Hksp = arrays['Hksp'].reshape((nawf,nawf,snktot,nspin))
+        Ef = E_Fermi(data_controller, Hksp, parallel=True)
+        dinds = np.diag_indices(nawf)
+        Hksp[dinds[0], dinds[1]] -= Ef
+        arrays['Hksp'] = np.moveaxis(Hksp, 2, 0)
+      else:
+        arrays['Hksp'] = np.reshape(np.moveaxis(arrays['Hksp'],0,1), (snktot,nawf,nawf,nspin))
 
       get_K_grid_fft(self.data_controller)
 
@@ -755,10 +845,10 @@ mo    '''
           print('d : nk -> nfft\n1 : %d -> %d\n2 : %d -> %d\n3 : %d -> %d'%(nko1,nfft1,nko2,nfft2,nko3,nfft3))
         print('New estimated maximum array size: %.2f GBytes'%gbyte)
 
-    except:
+    except Exception as e:
       self.report_exception('interpolated_hamiltonian')
       if attr['abort_on_exception']:
-        self.comm.Abort()
+        raise e
 
     self.report_module_time('R -> k with Zero Padding')
 
@@ -767,7 +857,7 @@ mo    '''
   def pao_eigh ( self, bval=0 ):
     '''
     Calculate the Eigen values and vectors of k-space Hamiltonian 'Hksp'
-    Yields 'E_k' and 'v_k'
+    Populates DataController with 'E_k' and 'v_k'
 
     Arguments:
         bval (int): Top valence band number (nelec/2) to correctly shift Eigenvalues
@@ -783,7 +873,8 @@ mo    '''
     if 'bval' not in attr: attr['bval'] = bval
 
     # HRs and Hks are replaced with Hksp
-    del arrays['HRs']
+    if 'HRs' in arrays:
+      del arrays['HRs']
 
     try:
       if 'Hksp' not in arrays:
@@ -809,10 +900,10 @@ mo    '''
           arrays['E_k'] -= np.amax(arrays['E_k'][:,attr['bval'],:])
         self.comm.Barrier()
         arrays['E_k'] = scatter_full(arrays['E_k'], attr['npool'])
-    except:
+    except Exception as e:
       self.report_exception('pao_eigh')
       if attr['abort_on_exception']:
-        self.comm.Abort()
+        raise e
 
     self.report_module_time('Eigenvalues')
 
@@ -822,7 +913,7 @@ mo    '''
     '''
     Calculate the Gradient of the k-space Hamiltonian, 'Hksp'
     Requires 'Hksp'
-    Yields 'dHksp'
+    Populates DataController with 'dHksp'
 
     Arguments:
       None
@@ -868,10 +959,10 @@ mo    '''
         # No more need for k-space Hamiltonian
         del arrays['Hksp']
       
-    except:
+    except Exception as e:
       self.report_exception('gradient_and_momenta')
       if attr['abort_on_exception']:
-        self.comm.Abort()
+        raise e
 
     self.report_module_time('Gradient')
 
@@ -885,7 +976,7 @@ mo    '''
   def adaptive_smearing ( self, smearing='gauss' ):
     '''
     Calculate the Adaptive Smearing parameters
-    Yields 'deltakp' and 'deltakp2'
+    Populates DataController with 'deltakp' and 'deltakp2'
 
     Arguments:
         smearing (str): Smearing type (m-p and gauss)
@@ -902,10 +993,10 @@ mo    '''
 
     try:
       do_adaptive_smearing(self.data_controller, smearing)
-    except:
+    except Exception as e:
       self.report_exception('adaptive_smearing')
       if attr['abort_on_exception']:
-        self.comm.Abort()
+        raise e
     self.report_module_time('Adaptive Smearing')
 
 
@@ -951,18 +1042,36 @@ mo    '''
         if do_pdos:
           from .defs.do_pdos import do_pdos_adaptive
           do_pdos_adaptive(self.data_controller, emin, emax, ne)
-
-    except:
+    except Exception as e:
       self.report_exception('dos')
       if attr['abort_on_exception']:
-        self.comm.Abort()
+        raise e
 
     mname = 'DoS%s'%('' if attr['smearing'] is None else ' (Adaptive Smearing)')
     self.report_module_time(mname)
 
+
+
+  def density ( self, nr1=48, nr2=48, nr3=48):
+    '''
+    Calculate the Electron Density in real space
+    Arguments:
+        nr1,nr2,nr3: real space grid
+
+    Returns:
+        None
+    '''
+    from .defs.do_real_space import do_density
+
+    do_density(self.data_controller, nr1,nr2,nr3)
+    
+    self.report_module_time('Density')
+
+
+  
   def trim_non_projectable_bands ( self ):
 
-    arrays = self.data_controller.data_arrays
+    arrays, attributes = self.data_controller.data_dicts()
 
     bnd = attributes['nawf'] = attributes['bnd']
 
@@ -994,10 +1103,10 @@ mo    '''
 
     try:
       do_fermisurf(self.data_controller)
-    except:
+    except Exception as e:
       self.report_exception('fermi_surface')
       if attr['abort_on_exception']:
-        self.comm.Abort()
+        raise e
 
     self.report_module_time('Fermi Surface')
 
@@ -1030,10 +1139,10 @@ mo    '''
       else:
         if self.rank == 0:
           print('Cannot compute spin texture with nspin=2')
-    except:
+    except Exception as e:
       self.report_exception('spin_texture')
       if attr['abort_on_exception']:
-        self.comm.Abort()
+        raise e
 
 
     self.comm.Barrier()
@@ -1073,10 +1182,10 @@ mo    '''
 
     try:
       do_spin_Hall(self.data_controller, twoD, do_ac)
-    except:
+    except Exception as e:
       self.report_exception('spin_Hall')
       if attr['abort_on_exception']:
-        self.comm.Abort()
+        raise e
 
     self.report_module_time('Spin Hall Conductivity')
 
@@ -1110,16 +1219,43 @@ mo    '''
 
     try:
       do_anomalous_Hall(self.data_controller, do_ac)
-    except:
+    except Exception as e:
       self.report_exception('anomalous_Hall')
       if attr['abort_on_exception']:
-        self.comm.Abort()
+        raise e
 
     self.report_module_time('Anomalous Hall Conductivity')
 
 
+  def effective_mass ( self, emin=-1., emax=1., ne=1000 ):
 
-  def doping( self, tmin=300, tmax=300, nt=1, delta=0.01, emin=-1., emax=1., ne=1000, doping_conc=0., core_electrons=0., fname='doping_' ):
+    '''
+    Calculate effective mass components for kx,ky and kz
+
+    Arguments:
+        tmin (float): The minimum temperature in the range
+        tmax (float): The maximum temperature in the range
+        nt (float): The number of temperature increments
+        emin (float): The minimum energy in the range
+        emax (float): The maximum energy in the range
+        ne (float): The number of energy increments
+
+    Returns:
+        None
+    '''
+    from .defs.do_effective_mass import do_effective_mass
+
+    arrays,attr = self.data_controller.data_dicts()
+
+    ene = np.linspace(emin, emax, ne)
+
+    do_effective_mass(self.data_controller, ene)
+
+    self.report_module_time('Effective mass')
+
+
+
+  def doping ( self, tmin=300, tmax=300, nt=1, delta=0.01, emin=-1., emax=1., ne=1000, doping_conc=0., core_electrons=0., fname='doping_' ):
     '''
     Calculate the chemical potential that corresponds to specified doping for different temperatures
 
@@ -1158,7 +1294,7 @@ mo    '''
 
 
 
-  def transport ( self, tmin=300., tmax=300., nt=1, emin=1., emax=10., ne=500, scattering_channels=[], scattering_weights=[], tau_dict={}, write_to_file=True, save_tensors=False ):
+  def transport ( self, tmin=300., tmax=300., nt=1, emin=-2., emax=2., ne=500, scattering_channels=[], scattering_weights=[], tau_dict={}, do_hall=False, write_to_file=True, save_tensors=False ):
     '''
     Calculate the Transport Properties
 
@@ -1173,6 +1309,7 @@ mo    '''
         scattering_channels (list): List of strings specifying a built in model and/or TauModel objects to evaluate
         scattering_weights (list): List of coefficients weighting components of the harmonic sum when computing tau
         tau_dict (dict): Dictionary housing parameters required for calculating Tau with built-in models
+        do_hall (bool): Set True to calculate hall coefficient
         write_to_file (bool): Set True to write tensors to file
         save_tensors (bool): Set True to save the tensors into the data controller
 
@@ -1194,12 +1331,12 @@ mo    '''
       for n in range(bnd):
         velkp[:,:,n,:] = np.real(arrays['pksp'][:,:,n,n,:])
 
-      do_transport(self.data_controller, temps, ene, velkp, sc, sw, write_to_file, save_tensors)
+      do_transport(self.data_controller, temps, ene, velkp, sc, sw, do_hall, write_to_file, save_tensors)
 
-    except:
+    except Exception as e:
       self.report_exception('transport')
       if attr['abort_on_exception']:
-        self.comm.Abort()
+        raise e
 
     self.report_module_time('Transport')
 
@@ -1240,26 +1377,151 @@ mo    '''
     try:
       ene = np.linspace(emin, emax, ne)
       do_dielectric_tensor(self.data_controller, ene)
-    except:
+    except Exception as e:
       self.report_exception('dielectric_tensor')
       if attr['abort_on_exception']:
-        self.comm.Abort()
+        raise e
 
     self.report_module_time('Dielectric Tensor')
 
 
-  def find_weyl_points ( self, symmetrize=None, search_grid=[8,8,8] ):
+  def find_weyl_points ( self, symmetrize=None, test_rad=0.01, search_grid=[8,8,8] ):
     from .defs.do_find_Weyl import find_weyl
 
     try:
       if symmetrize is not None:
         self.data_controller.data_attributes['symmetrize'] = symmetrize
-      find_weyl(self.data_controller, search_grid)
+      find_weyl(self.data_controller, test_rad, search_grid)
 
-    except:
+    except Exception as e:
       self.report_exception('pao_hamiltonian')
       if self.data_controller.data_attributes['abort_on_exception']:
-        self.comm.Abort()
+        raise e
 
 
     self.report_module_time('Weyl Search')
+
+  def ipr(self, fname='ipr'):
+    '''
+    Compute the inverse partiticipation ratio (IPR) from PAO eigenstates
+
+                 \sum_n |v_nk|^4
+    IPR_nk = -----------------------
+              ( \sum_n |v_nk|^2 )^2
+
+    where n is the band index and k the k-point
+
+    The final shape is (nspin,nkpts,nbands,3),
+    where the last axis gives: 0 as the k-point coordinate,
+                               1 the band energy, and
+                               2 the inverse partition ratio
+
+    The result in saved to a ipr.npy file.
+    To open the file one should use:
+    `ipr = np.load('ipr.npy')`
+
+    '''
+
+    from .defs.do_ipr import inverse_participation_ratio
+    from os.path import join
+
+    arry, attr = self.data_controller.data_dicts()
+
+    try:
+      arry['ipr'] = inverse_participation_ratio(self.data_controller)
+      np.save(join(attr['opath'],fname+'.npy'),arry['ipr'])
+
+    except Exception as e:
+      self.report_exception('ipr')
+      if attr['abort_on_exception']:
+        raise e
+
+    self.report_module_time('Inverse Participation Ratio (IPR)')
+
+  def berry_phase (self, kspace_method='path', berry_path=None, high_sym_points=None, kpath_funct=None, nk1=100, nk2=100, closed=True, method='berry', sub=None, occupied=True, kradius=None, kcenter=[0.,0.,0.],kxlim=(-0.5,0.5),kylim=(-0.5,0.5),eigvals=False,fname='berry_phase',contin=False):
+    '''
+    Compute the Berry phase using the discretized formula.
+    See R. Resta, Rev. Mod. Phys. 66, 899 (1994) and R. Resta, J. Phys.: Condens. Matter 22, 123201 (2010)
+
+    Arguments:
+        kspace_method (str): method used to sample the BZ:
+                             *'path': 1D path along the BZ;
+                             *'track': 1D path along x direction for several points in the y direction;
+                             *'circle': circular path around a k-point, given center and radius;
+                             *'square': retangular region with nk1 points along x and nk2 points along y. Region defined given x and y start and end points or full BZ.
+        berry_path (str): A string representing the band path to follow. The first and last k-point must not be the same.
+        berry_high_sym_points (dictionary): A dictionary with symbols of high symmetry points as keys and length 3 numpy arrays containg the location of the symmetry points as values.
+        nk (int): Number of k-points to include in the path
+        closed (bool, optional): whether or not to include the connection of the last and first points in the loop
+        method (str, {'berry','zak'}): 'berry' returns the usual berry phase. 'zak' includes the Zak phase for 1D systems which takes into account the Bloch factor exp(-iG.r)
+                                        accumulated over a Brillouin zone. See J. Zak, Phys. Rev. Lett. 62, 2747 (1989)
+        sub (None or list of int, optional): index of selected bands to calculate the Berry phase
+        occupied (bool, optional): calculate the Berry phase over all occupied bands (if set to True, sub is set to None)
+        kxlim (tuple, float): start and end points in x direction for sampling the BZ, used when kspace_method='square' .
+        kylim (tuple, float): start and end points in y direction for sampling the BZ, used when kspace_method='square' .
+        eigvals (bool, optional): write overlap matrix eigenvalues
+        eigvecs (bool, optional): write overlap matrix eigenvectors
+        save_prd (bool, optional): save overlap matrix
+        fname (str, optional): filename to save the berry phase results
+        contin (bool, optional): make berry phase continuous by fixing 2pi shifts
+
+    Returns:
+        Berry/Zak phase
+
+    '''
+    from .defs.do_berry_phase import do_berry_phase
+
+    arry,attr = self.data_controller.data_dicts()
+
+    kspace_method = kspace_method.lower()
+    attr['berry_kspace_method'] = kspace_method
+
+    attr['berry_path'] = berry_path
+    arry['berry_high_sym_points'] = high_sym_points
+
+    attr['berry_nk'] = nk1
+    attr['berry_nk1'] = nk1
+    attr['berry_nk2'] = nk2
+
+    attr['berry_kpath_funct'] = kpath_funct
+
+    arry['berry_kxlim'] = kxlim
+    arry['berry_kylim'] = kylim
+
+    if kspace_method != 'square':
+      attr['berry_eigvals'] = eigvals
+    else:
+      attr['berry_eigvals'] = False
+
+    attr['berry_kradius'] = kradius
+    arry['berry_kcenter'] = np.array(kcenter)
+    attr['berry_eigvals'] = eigvals
+
+    method = method.lower()
+    if method in ['berry','zak']:
+      attr['berry_method'] = method
+    else:
+      print('method should be either berry or zak. Falling back to method = \'berry\'')
+      attr['berry_method'] = 'berry'
+
+    arry['berry_sub'] = sub
+    if sub != None or (sub == None and not occupied):
+      attr['berry_occupied'] = False
+    else:
+      attr['berry_occupied'] = occupied
+
+    attr['berry_closed'] = closed
+    attr['berry_contin'] = contin
+    attr['berry_fname'] = fname
+
+    try:
+
+      do_berry_phase(self)
+
+    except Exception as e:
+      self.report_exception('berry_phase')
+      if attr['abort_on_exception']:
+        raise e
+
+    self.report_module_time('Berry phase')
+
