@@ -130,12 +130,24 @@ def do_build_pao_hamiltonian ( data_controller ):
     from .pao_sym import open_grid_wrapper
     open_grid_wrapper(data_controller)
 
+  attr['nkpnts'] = nkpnts = np.prod(ashape[2:5])
+
   # NOTE: Take care of non-orthogonality, if needed
   # Hks from projwfc is orthogonal. If non-orthogonality is required, we have to 
   # apply a basis change to Hks as Hks -> Sks^(1/2)+*Hks*Sks^(1/2)
   # acbn0 flag == 0 - makes H non orthogonal (original basis of the atomic pseudo-orbitals)
   # acbn0 flag == 1 - makes H orthogonal (rotated basis) 
 
+  if rank == 0:
+    from .do_Efermi import E_Fermi
+    arry['Hks'] = np.reshape(arry['Hks'], ashape)
+
+    # Shift the Fermi energy to zero
+    tshape = (ashape[0],ashape[1],nkpnts,ashape[5])
+    Ef = E_Fermi(arry['Hks'].reshape(tshape), data_controller)
+    dinds = np.diag_indices(attr['nawf'])
+    arry['Hks'][dinds[0],dinds[1]] -= Ef
+
   if attr['acbn0']:
     if rank == 0:
       from .do_non_ortho import do_non_ortho
@@ -148,11 +160,7 @@ def do_build_pao_hamiltonian ( data_controller ):
 
       data_controller.write_Hk_acbn0()
     comm.Barrier()
-    quit()
 
-  if rank == 0:
-    arry['Hks'] = np.reshape(arry['Hks'], ashape)
-  attr['nkpnts'] = np.prod(ashape[2:5])
 
 
 def do_Hks_to_HRs ( data_controller ):
