@@ -756,122 +756,6 @@ def get_inv_op(shells):
 ############################################################################################
 ############################################################################################
 
-def read_shell ( workpath,savedir,species,atoms,spin_orb=False):
-    # reads in shelks from pseudo files
-    from os.path import join,exists
-    import numpy as np
-
-    # Get Shells for each species
-    sdict = {}
-    jchid = {}
-    jchia = None
-
-    for s in species:
-      sdict[s[0]],jchid[s[0]] = read_pseudopotential(join(workpath,savedir,s[1]))
-
-    #double the l=0 if spin orbit
-    if spin_orb:
-        for s,p in sdict.items():
-            tmp_list=[]
-            for o in p:
-                tmp_list.append(o)
-                # if l=0 include it twice
-                if o==0 or len(jchid[s])==0:
-                    tmp_list.append(o)
-            
-            sdict[s] = np.array(tmp_list)
-
-            # when using scalar rel pseido with spin orb..
-            if len(jchid[s])==0:
-                tmp=[]
-                for o in sdict[s][::2]:
-                    if o==0:
-                        tmp.extend([0.5])
-                    if o==1:
-                        tmp.extend([0.5,1.5])
-                    if o==2:
-                        tmp.extend([1.5,2.5])
-                    if o==3:
-                        tmp.extend([2.5,3.5])
-                jchid[s]=np.array(tmp)
-
-
-        jchia = np.hstack([jchid[a] for a in atoms])
-
-
-    # index of which orbitals belong to which atom in the basis
-    a_index = np.hstack([[a]*np.sum((2*sdict[atoms[a]])+1) for a in range(len(atoms))])
-
-    # value of l
-    shell   = np.hstack([sdict[a] for a in atoms])
-    return shell,a_index,jchia
-
-############################################################################################
-############################################################################################
-############################################################################################
-
-def read_pseudopotential ( fpp ):
-  '''
-  Reads a psuedopotential file to determine the included shells and occupations.
-
-  Arguments:
-      fnscf (string) - Filename of the pseudopotential, copied to the .save directory
-
-  Returns:
-      sh, nl (lists) - sh is a list of orbitals (s-0, p-1, d-2, etc)
-                       nl is a list of occupations at each site
-      sh and nl are representative of one atom only
-  '''
-
-  import numpy as np
-  import xml.etree.cElementTree as ET
-  import re
-
-  sh = []
-  # fully rel case
-  jchi=[]
-
-  # clean xnl before reading
-  with open(fpp) as ifo:
-      temp_str=ifo.read()
-
-  temp_str = re.sub('&',' ',temp_str)
-  f = NamedTemporaryFile(mode='w',delete=True)
-  f.write(temp_str)
-
-  try:
-      iterator_obj = ET.iterparse(f.name,events=('start','end'))
-      iterator     = iter(iterator_obj)
-      event,root   = next(iterator)
-
-      for event,elem in iterator:        
-          try:
-              for i in elem.findall("PP_PSWFC/"):
-                  sh.append(int(i.attrib['l']))
-          except Exception as e:
-              pass
-          for i in elem.findall("PP_SPIN_ORB/"):
-              try:
-                  jchi.append(float(i.attrib["jchi"]))
-              except: pass
-              
-      jchi = np.array(jchi)
-      sh   = np.array(sh)
-
-  except Exception as e:
-
-      with open(fpp) as ifo:
-          ifs=ifo.read()
-      res=re.findall("(.*)\s*Wavefunction",ifs)[1:]      
-      sh=np.array(list(map(int,list([x.split()[1] for x in res]))))
-
-
-  return sh,jchi
-
-############################################################################################
-############################################################################################
-############################################################################################
-
 def enforce_t_rev(Hksp_s,nk1,nk2,nk3,spin_orb,U_inv,jchia):
     # enforce time reversal symmetry on H(k)
     nawf=Hksp_s.shape[1]
@@ -927,6 +811,7 @@ def apply_t_rev(Hksp,kp,spin_orb,U_inv,jchia):
             if not spin_orb:
                 new_Hk_list.append(np.conj(Hksp[i]))
             else:
+#                print(U_inv.shape,U_TR.shape,Hksp.shape)
                 new_Hk_list.append(np.conj(U_inv*(U_TR @ Hksp[i] @ np.conj(U_TR.T))))
 
     if len(new_kp_list)==0:
@@ -1026,7 +911,7 @@ def open_grid(Hksp,full_grid,kp,symop,symop_cart,atom_pos,shells,a_index,equiv_a
 
 
     nawf = Hksp.shape[1]
-
+#    print(shells)
     # get inversion operator
     U_inv = get_inv_op(shells)
 
@@ -1225,11 +1110,12 @@ def open_grid(Hksp,full_grid,kp,symop,symop_cart,atom_pos,shells,a_index,equiv_a
 
     
     # for debugging purposes
-    try:
-        if rank==0:
-            check(Hksp,si_per_k,new_k_ind,orig_k_ind,phase_shifts,U,
-                  a_index,inv_flag,equiv_atom,kp,symop,full_grid,sym_info,sym_TR)        
-    except: pass
+    #try:
+    #    if rank==0:
+    #        check(Hksp,si_per_k,new_k_ind,orig_k_ind,phase_shifts,U,
+    #              a_index,inv_flag,equiv_atom,kp,symop,full_grid,sym_info,sym_TR)        
+    #except Exception as e:
+    #    print(('Exception in pao_sym, check.', e), flush=True)
 
     return Hksp
 
@@ -1299,18 +1185,25 @@ def open_grid_wrapper(data_controller):
     # get full grid in crystal fractional coords
     full_grid = get_full_grid(nk1,nk2,nk3)
 
-    # get shells and atom indices for blocks of the hamiltonian
-    shells,a_index,jchia = read_shell(data_attr['workpath'],data_attr['savedir'],
-                                      data_arrays['species'],atom_lab,
-                                      spin_orb=spin_orb)
-
-
-    kp_red = correct_roundoff_kp(kp_red,full_grid)
-
+    jchia = []
+    shells = []
+    a_index = []
+    sh = data_arrays['shells']
+    for i,a in enumerate(atom_lab):
+      ash = []
+      for v in sh[a]:
+        ash += [v,v] if v==0 and spin_orb else [v]
+      shells += ash
+      a_index += [i]*np.sum([2*n+1 for n in ash])
+      if spin_orb:
+        jchia += data_arrays['jchia'][a]
 
+    jchia = np.array(jchia)
+    shells = np.array(shells)
+    a_index = np.array(a_index)
 
     # correct small differences due to conversion
-
+    kp_red = correct_roundoff_kp(kp_red,full_grid)
 
     # we wont need this for now
     if rank==0:
@@ -1340,8 +1233,6 @@ def open_grid_wrapper(data_controller):
         else:
             Hksp=None
 
-    if rank==0:
-        data_arrays['Hks'] = reshift_efermi(data_arrays['Hks'],1,nelec,spin_orb)
 
 ############################################################################################
 ############################################################################################
@@ -1440,37 +1331,3 @@ def correct_roundoff_kp(kp,full_grid):
 ############################################################################################
 ############################################################################################
 ############################################################################################
-
-def reshift_efermi(Hksp,npool,nelec,spin_orb):
-
-#    Hksp=scatter_full(Hksp,npool)
-    nspin = Hksp.shape[-1]
-
-    eig = np.zeros((Hksp.shape[1],Hksp.shape[2],Hksp.shape[3]))
-
-    dinds = np.diag_indices(Hksp.shape[1])
-
-    for ispin in range(Hksp.shape[3]):
-        for kp in range(Hksp.shape[2]):
-            eig[:,kp,ispin] = LA.eigvalsh(Hksp[:,:,kp,ispin])
-
-#    eigs=gather_full(eig,npool)
-    
-
-    nbnd=nelec
-    if not spin_orb:
-        nbnd=np.floor(nelec/2.0)
-    
-    if rank==0:
-        nk=eig.shape[1]
-        eig=np.sort(np.ravel(eig))
-        efermi=eig[int(nk*nbnd*nspin-1)]
-    else:
-        efermi=None
-
-#    efermi = comm.bcast(efermi)
-
-    Hksp[dinds[0],dinds[1]] -= efermi
-#    Hksp=gather_full(Hksp,npool)
-    
-    return Hksp
